@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/access/transam/xact.c,v 1.156.2.1 2004/08/11 04:08:00 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/access/transam/xact.c,v 1.156.2.3 2006/05/21 20:06:43 tgl Exp $
  *
  * NOTES
  *		Transaction aborts can now occur two ways:
@@ -916,6 +916,11 @@ CommitTransaction(void)
 	if (s->state != TRANS_INPROGRESS)
 		elog(WARNING, "CommitTransaction and not in in-progress state");
 
+	/*
+	 * Do pre-commit processing (most of this stuff requires database
+	 * access, and in fact could still cause an error...)
+	 */
+
 	/*
 	 * Tell the trigger manager that this transaction is about to be
 	 * committed. He'll invoke all trigger deferred until XACT before we
@@ -923,28 +928,15 @@ CommitTransaction(void)
 	 */
 	DeferredTriggerEndXact();
 
+	/* Close open cursors */
+	AtCommit_Portals();
+
 	/*
-	 * Similarly, let ON COMMIT management do its thing before we start to
-	 * commit.
+	 * Let ON COMMIT management do its thing (must happen after closing
+	 * cursors, to avoid dangling-reference problems)
 	 */
 	PreCommit_on_commit_actions();
 
-	/* Prevent cancel/die interrupt while cleaning up */
-	HOLD_INTERRUPTS();
-
-	/*
-	 * set the current transaction state information appropriately during
-	 * the abort processing
-	 */
-	s->state = TRANS_COMMIT;
-
-	/*
-	 * Do pre-commit processing (most of this stuff requires database
-	 * access, and in fact could still cause an error...)
-	 */
-
-	AtCommit_Portals();
-
 	/* handle commit for large objects [ PA, 7/17/98 ] */
 	/* XXX probably this does not belong here */
 	lo_commit(true);
@@ -955,6 +947,15 @@ CommitTransaction(void)
 	/* Update the flat password file if we changed pg_shadow or pg_group */
 	AtEOXact_UpdatePasswordFile(true);
 
+	/* Prevent cancel/die interrupt while cleaning up */
+	HOLD_INTERRUPTS();
+
+	/*
+	 * set the current transaction state information appropriately during
+	 * the abort processing
+	 */
+	s->state = TRANS_COMMIT;
+
 	/*
 	 * Here is where we really truly commit.
 	 */
@@ -1053,9 +1054,14 @@ AbortTransaction(void)
 
 	/*
 	 * check the current transaction state
+	 *
+	 * reduced to DEBUG2 because this is expected when rejecting an
+	 * invalidly-encoded query outside a transaction block.  PG 8.0
+	 * and up fix it better, but it's not worth back-porting those
+	 * changes to 7.4.
 	 */
 	if (s->state != TRANS_INPROGRESS)
-		elog(WARNING, "AbortTransaction and not in in-progress state");
+		elog(DEBUG2, "AbortTransaction and not in in-progress state");
 
 	/*
 	 * set the current transaction state information appropriately during
